@@ -15,7 +15,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD %iptcCharset);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.47';
+$VERSION = '1.56';
 
 %iptcCharset = (
     "\x1b%G"  => 'UTF8',
@@ -42,6 +42,15 @@ my %isStandardIPTC = (
     'MIE-IPTC'                  => 1,
     'EPS-Photoshop-IPTC'        => 1,
     'PS-Photoshop-IPTC'         => 1,
+    'EXV-APP13-Photoshop-IPTC'  => 1,
+    # set file types to 0 if they have a standard location
+    JPEG => 0,
+    TIFF => 0,
+    PSD  => 0,
+    MIE  => 0,
+    EPS  => 0,
+    PS   => 0,
+    EXV  => 0,
 );
 
 my %fileFormat = (
@@ -193,7 +202,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     80 => {
         Name => 'TimeSent',
@@ -202,7 +211,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     90 => {
         Name => 'CodedCharacterSet',
@@ -219,6 +228,7 @@ my %fileFormat = (
         },
         Protected => 1,
         Format => 'string[0,32]',
+        ValueConvInv => '$val =~ /^UTF-?8$/i ? "\x1b%G" : $val',
         # convert ISO 2022 escape sequences to a more readable format
         PrintConv => \&PrintCodedCharset,
         PrintConvInv => \&PrintInvCodedCharset,
@@ -330,7 +340,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     35 => {
         Name => 'ReleaseTime',
@@ -339,7 +349,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     37 => {
         Name => 'ExpirationDate',
@@ -348,7 +358,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     38 => {
         Name => 'ExpirationTime',
@@ -357,7 +367,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     40 => {
         Name => 'SpecialInstructions',
@@ -387,7 +397,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     50 => {
         Name => 'ReferenceNumber',
@@ -401,7 +411,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     60 => {
         Name => 'TimeCreated',
@@ -410,7 +420,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     62 => {
         Name => 'DigitalCreationDate',
@@ -419,7 +429,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcDate($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     63 => {
         Name => 'DigitalCreationTime',
@@ -428,7 +438,7 @@ my %fileFormat = (
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => 'Image::ExifTool::IPTC::IptcTime($val)',
-        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($val)',
+        PrintConvInv => 'Image::ExifTool::IPTC::InverseDateOrTime($self,$val)',
     },
     65 => {
         Name => 'OriginatingProgram',
@@ -524,7 +534,7 @@ my %fileFormat = (
             I haven't found a reference for the format of tags 121, 184-188 and
             225-232, so I have just make them writable as strings with
             reasonable length.  Beware that if this is wrong, other utilities
-            won't be able to read these tags as written by ExifTool
+            may not be able to read these tags as written by ExifTool
         },
     },
     122 => {
@@ -629,7 +639,7 @@ my %fileFormat = (
     },
     202 => {
         Name => 'ObjectPreviewData',
-        Groups => { 2 => 'Image' },
+        Groups => { 2 => 'Preview' },
         Format => 'undef[0,256000]',
         Binary => 1,
     },
@@ -664,6 +674,8 @@ my %fileFormat = (
     231 => {
         Name => 'DocumentHistory',
         Format => 'string[0,256]', # (guess)
+        ValueConv => '$val =~ s/\0+/\n/g; $val', # (have seen embedded nulls)
+        ValueConvInv => '$val',
     },
     232 => {
         Name => 'ExifCameraInfo',
@@ -976,18 +988,18 @@ sub PrintCodedCharset($)
 # Returns: IPTC character set if translation required (or 'bad' if unknown)
 sub HandleCodedCharset($$)
 {
-    my ($exifTool, $val) = @_;
+    my ($et, $val) = @_;
     my $xlat = $iptcCharset{$val};
     unless ($xlat) {
         if ($val =~ /^\x1b\x25/) {
             # some unknown character set invoked
             $xlat = 'bad';  # flag unsupported coding
         } else {
-            $xlat = $exifTool->Options('CharsetIPTC');
+            $xlat = $et->Options('CharsetIPTC');
         }
     }
     # no need to translate if Charset is the same
-    undef $xlat if $xlat eq $exifTool->Options('Charset');
+    undef $xlat if $xlat eq $et->Options('Charset');
     return $xlat;
 }
 
@@ -998,28 +1010,31 @@ sub HandleCodedCharset($$)
 # Updates value on return
 sub TranslateCodedString($$$$)
 {
-    my ($exifTool, $valPtr, $xlatPtr, $read) = @_;
+    my ($et, $valPtr, $xlatPtr, $read) = @_;
     if ($$xlatPtr eq 'bad') {
-        $exifTool->Warn('Some IPTC characters not converted (unsupported CodedCharacterSet)');
+        $et->Warn('Some IPTC characters not converted (unsupported CodedCharacterSet)');
         undef $$xlatPtr;
     } elsif (not $read) {
-        $$valPtr = $exifTool->Decode($$valPtr, undef, undef, $$xlatPtr);
+        $$valPtr = $et->Decode($$valPtr, undef, undef, $$xlatPtr);
     } elsif ($$valPtr !~ /[\x14\x15\x1b]/) {
-        $$valPtr = $exifTool->Decode($$valPtr, $$xlatPtr);
+        $$valPtr = $et->Decode($$valPtr, $$xlatPtr);
     } else {
         # don't yet support reading ISO 2022 shifted character sets
-        $exifTool->WarnOnce('Some IPTC characters not converted (ISO 2022 shifting not supported)');
+        $et->WarnOnce('Some IPTC characters not converted (ISO 2022 shifting not supported)');
     }
 }
 
 #------------------------------------------------------------------------------
 # Is this IPTC in a standard location?
 # Inputs: 0) Current metadata path string
-# Returns: true if path is standard
+# Returns: true if path is standard, 0 if file type doesn't have standard IPTC,
+#          or undef if IPTC is non-standard
 sub IsStandardIPTC($)
 {
     my $path = shift;
-    return $isStandardIPTC{$path};
+    return 1 if $isStandardIPTC{$path};
+    return 0 unless $path =~ /^(\w+)/ and defined $isStandardIPTC{$1};
+    return undef;   # non-standard
 }
 
 #------------------------------------------------------------------------------
@@ -1029,52 +1044,66 @@ sub IsStandardIPTC($)
 # Returns: 1 on success, 0 otherwise
 sub ProcessIPTC($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $pos = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen} || 0;
     my $dirEnd = $pos + $dirLen;
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my $success = 0;
     my ($lastRec, $recordPtr, $recordName);
 
-    $verbose and $dirInfo and $exifTool->VerboseDir('IPTC', 0, $$dirInfo{DirLen});
+    $verbose and $dirInfo and $et->VerboseDir('IPTC', 0, $$dirInfo{DirLen});
 
     if ($tagTablePtr eq \%Image::ExifTool::IPTC::Main) {
-        # calculate MD5 if Digest::MD5 is available (for standard IPTC only)
-        my $path = $exifTool->MetadataPath();
-        if (IsStandardIPTC($path)) {
-            my $md5;
-            if (eval 'require Digest::MD5') {
-                if ($pos or $dirLen != length($$dataPt)) {
-                    $md5 = Digest::MD5::md5(substr $$dataPt, $pos, $dirLen);
+        my $path = $et->MetadataPath();
+        my $isStd = IsStandardIPTC($path);
+        if (defined $isStd and not $$et{DIR_COUNT}{STD_IPTC}) {
+            # set flag to ensure we only have one family 1 "IPTC" group
+            $$et{DIR_COUNT}{STD_IPTC} = 1;
+            # calculate MD5 if Digest::MD5 is available (truly standard IPTC only)
+            if ($isStd) {
+                my $md5;
+                if (eval { require Digest::MD5 }) {
+                    if ($pos or $dirLen != length($$dataPt)) {
+                        $md5 = Digest::MD5::md5(substr $$dataPt, $pos, $dirLen);
+                    } else {
+                        $md5 = Digest::MD5::md5($$dataPt);
+                    }
                 } else {
-                    $md5 = Digest::MD5::md5($$dataPt);
+                    # a zero digest indicates IPTC exists but we don't have Digest::MD5
+                    $md5 = "\0" x 16;
                 }
-            } else {
-                # a zero digest indicates IPTC exists but we don't have Digest::MD5
-                $md5 = "\0" x 16;
+                $et->FoundTag('CurrentIPTCDigest', $md5);
             }
-            $exifTool->FoundTag('CurrentIPTCDigest', $md5);
-        } elsif ($Image::ExifTool::MWG::strict and $$exifTool{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/) {
-            # ignore non-standard IPTC while in strict MWG compatibility mode
-            $exifTool->Warn("Ignored non-standard IPTC at $path");
-            return 1;
+        } else {
+            if (($Image::ExifTool::MWG::strict or $et->Options('Validate')) and
+                $$et{FILE_TYPE} =~ /^(JPEG|TIFF|PSD)$/)
+            {
+                if ($Image::ExifTool::MWG::strict) {
+                    # ignore non-standard IPTC while in strict MWG compatibility mode
+                    $et->Warn("Ignored non-standard IPTC at $path");
+                    return 1;
+                } else {
+                    $et->Warn("Non-standard IPTC at $path", 1);
+                }
+            }
+            # extract non-standard IPTC
+            my $count = ($$et{DIR_COUNT}{IPTC} || 0) + 1;  # count non-standard IPTC
+            $$et{DIR_COUNT}{IPTC} = $count;
+            $$et{LOW_PRIORITY_DIR}{IPTC} = 1;       # lower priority of non-standard IPTC
+            $$et{SET_GROUP1} = '+' . ($count + 1);  # add number to family 1 group name
         }
-        # set family 1 group name if multiple IPTC directories
-        my $dirCount = ($exifTool->{DIR_COUNT}->{IPTC} || 0) + 1;
-        $exifTool->{DIR_COUNT}->{IPTC} = $dirCount;
-        $exifTool->{SET_GROUP1} = '+' . $dirCount if $dirCount > 1;
     }
     # begin by assuming default IPTC encoding
-    my $xlat = $exifTool->Options('CharsetIPTC');
-    undef $xlat if $xlat eq $exifTool->Options('Charset');
+    my $xlat = $et->Options('CharsetIPTC');
+    undef $xlat if $xlat eq $et->Options('Charset');
 
     # quick check for improperly byte-swapped IPTC
     if ($dirLen >= 4 and substr($$dataPt, $pos, 1) ne "\x1c" and
                          substr($$dataPt, $pos + 3, 1) eq "\x1c")
     {
-        $exifTool->Warn('IPTC data was improperly byte-swapped');
+        $et->Warn('IPTC data was improperly byte-swapped');
         my $newData = pack('N*', unpack('V*', substr($$dataPt, $pos, $dirLen) . "\0\0\0"));
         $dataPt = \$newData;
         $pos = 0;
@@ -1082,13 +1111,13 @@ sub ProcessIPTC($$$)
         # NOTE: MUST NOT access $dirInfo DataPt, DirStart or DataLen after this!
     }
     # extract IPTC as a block if specified
-    if ($exifTool->{REQ_TAG_LOOKUP}{iptc} or ($exifTool->{OPTIONS}{Binary} and
-        not $exifTool->{EXCL_TAG_LOOKUP}{iptc}))
+    if ($$et{REQ_TAG_LOOKUP}{iptc} or ($$et{TAGS_FROM_FILE} and
+        not $$et{EXCL_TAG_LOOKUP}{iptc}))
     {
         if ($pos or $dirLen != length($$dataPt)) {
-            $exifTool->FoundTag('IPTC', substr($$dataPt, $pos, $dirLen));
+            $et->FoundTag('IPTC', substr($$dataPt, $pos, $dirLen));
         } else {
-            $exifTool->FoundTag('IPTC', $$dataPt);
+            $et->FoundTag('IPTC', $$dataPt);
         }
     }
     while ($pos + 5 <= $dirEnd) {
@@ -1101,7 +1130,7 @@ sub ProcessIPTC($$$)
                 my $remaining = substr($$dataPt, $pos, $dirEnd - $pos);
                 last unless $remaining =~ /[^\0]/;
             }
-            $exifTool->Warn(sprintf('Bad IPTC data tag (marker 0x%x)',$id));
+            $et->Warn(sprintf('Bad IPTC data tag (marker 0x%x)',$id));
             last;
         }
         $pos += 5;      # step to after field header
@@ -1109,7 +1138,7 @@ sub ProcessIPTC($$$)
         if ($len & 0x8000) {
             my $n = $len & 0x7fff; # get num bytes in length field
             if ($pos + $n > $dirEnd or $n > 8) {
-                $exifTool->VPrint(0, "Invalid extended IPTC entry (dataset $rec:$tag, len $len)\n");
+                $et->VPrint(0, "Invalid extended IPTC entry (dataset $rec:$tag, len $len)\n");
                 $success = 0;
                 last;
             }
@@ -1119,25 +1148,25 @@ sub ProcessIPTC($$$)
             }
         }
         if ($pos + $len > $dirEnd) {
-            $exifTool->VPrint(0, "Invalid IPTC entry (dataset $rec:$tag, len $len)\n");
+            $et->VPrint(0, "Invalid IPTC entry (dataset $rec:$tag, len $len)\n");
             $success = 0;
             last;
         }
         if (not defined $lastRec or $lastRec != $rec) {
             my $tableInfo = $tagTablePtr->{$rec};
             unless ($tableInfo) {
-                $exifTool->WarnOnce("Unrecognized IPTC record $rec (ignored)");
+                $et->WarnOnce("Unrecognized IPTC record $rec (ignored)");
                 $pos += $len;
                 next;   # ignore this entry
             }
             my $tableName = $tableInfo->{SubDirectory}->{TagTable};
             unless ($tableName) {
-                $exifTool->Warn("No table for IPTC record $rec!");
+                $et->Warn("No table for IPTC record $rec!");
                 last;   # this shouldn't happen
             }
             $recordName = $$tableInfo{Name};
             $recordPtr = Image::ExifTool::GetTagTable($tableName);
-            $exifTool->VPrint(0,$$exifTool{INDENT},"-- $recordName record --\n");
+            $et->VPrint(0,$$et{INDENT},"-- $recordName record --\n");
             $lastRec = $rec;
         }
         my $val = substr($$dataPt, $pos, $len);
@@ -1149,7 +1178,7 @@ sub ProcessIPTC($$$)
             AddTagToTable($recordPtr, $tag, { Unknown => 1 });
         }
 
-        my $tagInfo = $exifTool->GetTagInfo($recordPtr, $tag);
+        my $tagInfo = $et->GetTagInfo($recordPtr, $tag);
         my $format;
         # (could use $$recordPtr{FORMAT} if no Format below, but don't do this to
         #  be backward compatible with improperly written PhotoMechanic tags)
@@ -1171,11 +1200,11 @@ sub ProcessIPTC($$$)
                 $val =~ s/\0+$//;   # some braindead softwares add null terminators
                 if ($rec == 1) {
                     # handle CodedCharacterSet tag
-                    $xlat = HandleCodedCharset($exifTool, $val) if $tag == 90;
+                    $xlat = HandleCodedCharset($et, $val) if $tag == 90;
                 # translate characters if necessary and special characters exist
                 } elsif ($xlat and $rec < 7 and $val =~ /[\x80-\xff]/) {
                     # translate to specified character set
-                    TranslateCodedString($exifTool, \$val, \$xlat, 1);
+                    TranslateCodedString($et, \$val, \$xlat, 1);
                 }
             } elsif ($format =~ /^digits/) {
                 $val =~ s/\0+$//;
@@ -1183,7 +1212,7 @@ sub ProcessIPTC($$$)
                 warn("Invalid IPTC format: $format");
             }
         }
-        $verbose and $exifTool->VerboseInfo($tag, $tagInfo,
+        $verbose and $et->VerboseInfo($tag, $tagInfo,
             Table   => $tagTablePtr,
             Value   => $val,
             DataPt  => $dataPt,
@@ -1193,12 +1222,13 @@ sub ProcessIPTC($$$)
             Extra   => ", $recordName record",
             Format  => $format,
         );
-        $exifTool->FoundTag($tagInfo, $val) if $tagInfo;
+        $et->FoundTag($tagInfo, $val) if $tagInfo;
         $success = 1;
 
         $pos += $len;   # increment to next field
     }
-    delete $exifTool->{SET_GROUP1};
+    delete $$et{SET_GROUP1};
+    delete $$et{LOW_PRIORITY_DIR}{IPTC};
     return $success;
 }
 
@@ -1223,7 +1253,7 @@ image files.
 
 =head1 AUTHOR
 
-Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
